@@ -1,14 +1,14 @@
 #
 # mountall.sh	Mount all filesystems.
 #
-# Version:	@(#)mountall.sh  2.78  11-Feb-2000  miquels@cistron.nl
+# Version:	@(#)mountall.sh  2.78  05-Jun-2000  miquels@cistron.nl
 #
 . /etc/default/rcS
 
 #
 # Mount local file systems in /etc/fstab.
 #
-[ "$VERBOSE" != no ] && echo "Mounting local file systems..."
+[ "$VERBOSE" != no ] && echo "Mounting local filesystems..."
 mount -avt nonfs,noproc,nosmbfs
 
 #
@@ -25,5 +25,5 @@ kill -USR1 1
 # Execute swapon command again, in case we want to swap to
 # a file on a now mounted filesystem.
 #
-swapon -a 2>/dev/null
+grep -qs resync /proc/mdstat || swapon -a 2> /dev/null
 
