@@ -5,10 +5,10 @@
  *		     init [0123456SsQqAaBbCc]
  *		  telinit [0123456SsQqAaBbCc]
  *
- * Version:	@(#)init.c  2.76-4  24-Apr-1999  miquels@cistron.nl
+ * Version:	@(#)init.c  2.78  11-Feb-2000  miquels@cistron.nl
  *
  *		This file is part of the sysvinit suite,
- *		Copyright 1991-1999 Miquel van Smoorenburg.
+ *		Copyright 1991-2000 Miquel van Smoorenburg.
  *
  *		This program is free software; you can redistribute it and/or
  *		modify it under the terms of the GNU General Public License
@@ -30,6 +30,7 @@
 #include <sys/ioctl.h>
 #include <sys/wait.h>
 #include <sys/kd.h>
+#include <sys/resource.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <errno.h>
@@ -47,8 +48,13 @@
 
 #ifdef __i386__
 #  if (__GLIBC__ >= 2)
-#    include <sigcontext.h>
+     /* GNU libc 2.x */
+#    if (__GLIBC_MINOR__ == 0)
+       /* Only glibc 2.0 needs this */
+#      include <sigcontext.h>
+#    endif
 #  else
+     /* Linux libc5 */
 #    include <asm/sigcontext.h>
 #    define sigcontext sigcontext_struct
 #  endif
@@ -65,14 +71,18 @@
 #endif
 
 /* Set a signal handler. */
-#define SETSIG(sa, sig, fun)	sa.sa_handler = fun; \
-				sa.sa_flags = SA_RESTART; \
-				sigaction(sig, &sa, NULL);
+#define SETSIG(sa, sig, fun, flags) \
+		do { \
+			sa.sa_handler = fun; \
+			sa.sa_flags = flags; \
+			sigemptyset(&sa.sa_mask); \
+			sigaction(sig, &sa, NULL); \
+		} while(0)
 
 /* Version information */
-char *Version = "@(#) init 2.76-4  24-Apr-1999  miquels@cistron.nl";
-char *bootmsg = "version 2.76 booting";
-#define E_VERSION "INIT_VERSION=sysvinit-2.76"
+char *Version = "@(#) init 2.78  11-Feb-2000  miquels@cistron.nl";
+char *bootmsg = "version 2.78 %s";
+#define E_VERSION "INIT_VERSION=sysvinit-2.78"
 
 CHILD *family = NULL;		/* The linked list of all entries */
 CHILD *newFamily = NULL;	/* The list after inittab re-read */
@@ -91,10 +101,11 @@ char runlevel = 'S';		/* The current run level */
 char thislevel = 'S';		/* The current runlevel */
 char prevlevel = 'N';		/* Previous runlevel */
 int dfl_level = 0;		/* Default runlevel */
-int got_cont = 0;		/* Set if we received the SIGCONT signal */
+sig_atomic_t got_cont = 0;	/* Set if we received the SIGCONT signal */
+sig_atomic_t got_signals;	/* Set if we received a signal. */
 int emerg_shell = 0;		/* Start emergency shell? */
-unsigned int got_signals;	/* Set if we received a signal. */
-int wrote_reboot	= 1;	/* Set when we wrote the reboot record */
+int wrote_wtmp_reboot = 1;	/* Set when we wrote the reboot record */
+int wrote_utmp_reboot = 1;	/* Set when we wrote the reboot record */
 int sltime = 5;			/* Sleep time between TERM and KILL */
 char *argv0;			/* First arguments; show up in ps listing */
 int maxproclen;			/* Maximal length of argv[0] */
@@ -109,7 +120,7 @@ int main(int, char **);
 int reload = 0;			/* Should we do initialization stuff? */
 char *myname="/sbin/init";	/* What should we exec */
 int oops_error;			/* Used by some of the re-exec code. */
-char *Signature = "12567362";	/* Signature for re-exec fd */
+const char *Signature = "12567362";	/* Signature for re-exec fd */
 
 /* Macro to see if this is a special action */
 #define ISPOWER(i) ((i) == POWERWAIT || (i) == POWERFAIL || \
@@ -160,7 +171,8 @@ struct {
   { "-TL",	   D_THISLEVEL	},
   { "-PL",	   D_PREVLEVEL	},
   { "-SI",	   D_GOTSIGN	},
-  { "-WR",	   D_WROTE_REBOOT},
+  { "-WR",	   D_WROTE_WTMP_REBOOT},
+  { "-WU",	   D_WROTE_UTMP_REBOOT},
   { "-ST",	   D_SLTIME	},
   { "-DB",	   D_DIDBOOT	},
   { "",	   	   0		}
@@ -209,7 +221,8 @@ void send_state(int fd)
 	fprintf(fp, "-TL%c\n", thislevel);
 	fprintf(fp, "-PL%c\n", prevlevel);
 	fprintf(fp, "-SI%u\n", got_signals);
-	fprintf(fp, "-WR%d\n", wrote_reboot);
+	fprintf(fp, "-WR%d\n", wrote_wtmp_reboot);
+	fprintf(fp, "-WU%d\n", wrote_utmp_reboot);
 	fprintf(fp, "-ST%d\n", sltime);
 	fprintf(fp, "-DB%d\n", did_boot);
 
@@ -272,7 +285,7 @@ static int get_cmd(FILE *f)
 	char	cmd[4] = "   ";
 	int	i;
 
-	if (fread(cmd, 1, 3, f) != 3)
+	if (fread(cmd, 1, sizeof(cmd) - 1, f) != sizeof(cmd) - 1)
 		return C_EOF;
 
 	for(i = 0; cmds[i].cmd && strcmp(cmds[i].name, cmd) != 0; i++)
@@ -312,8 +325,11 @@ static CHILD *get_record(FILE *f)
 			case D_GOTSIGN:
 				fscanf(f, "%u\n", &got_signals);
 				break;
-			case D_WROTE_REBOOT:
-				fscanf(f, "%d\n", &wrote_reboot);
+			case D_WROTE_WTMP_REBOOT:
+				fscanf(f, "%d\n", &wrote_wtmp_reboot);
+				break;
+			case D_WROTE_UTMP_REBOOT:
+				fscanf(f, "%d\n", &wrote_utmp_reboot);
 				break;
 			case D_SLTIME:
 				fscanf(f, "%d\n", &sltime);
@@ -353,7 +369,7 @@ static CHILD *get_record(FILE *f)
 			get_string(p->process, sizeof(p->process), f);
 			break;
 		case C_FLAG:
-			get_string(s, 32, f);
+			get_string(s, sizeof(s), f);
 			for(i = 0; flags[i].name; i++) {
 				if (strcmp(flags[i].name,s) == 0)
 					break;
@@ -361,7 +377,7 @@ static CHILD *get_record(FILE *f)
 			p->flags |= flags[i].mask;
 			break;
 		case C_ACTION:
-			get_string(s, 32, f);
+			get_string(s, sizeof(s), f);
 			for(i = 0; actions[i].name; i++) {
 				if (strcmp(actions[i].name, s) == 0)
 					break;
@@ -391,7 +407,7 @@ int receive_state(int fd)
 
  	if (get_cmd(f) != C_VER)
 		return -1;
-	get_string(old_version, 256, f);
+	get_string(old_version, sizeof(old_version), f);
 	oops_error = 0;
 	for (pp = &family; (*pp = get_record(f)) != NULL; pp = &((*pp)->next))
 		;
@@ -464,7 +480,7 @@ void console_init(void)
  */
 int console_open(int mode)
 {
-	int f, fd;
+	int f, fd = -1;
 	int m;
 
 	/*
@@ -475,7 +491,7 @@ int console_open(int mode)
 	/*
 	 *	Retry the open five times.
 	 */
-	for(f = 0;f < 5; f++)
+	for(f = 0; f < 5; f++)
 		if ((fd = open(console_dev, m)) >= 0) break;
 
 	if (fd < 0) return fd;
@@ -501,8 +517,9 @@ void signal_handler(int sig)
  */
 void chld_handler()
 {
-	CHILD *ch;
-	int pid, st;
+	CHILD		*ch;
+	int		pid, st;
+	int		saved_errno = errno;
 
 	/*
 	 *	Find out which process(es) this was (were)
@@ -530,6 +547,7 @@ void chld_handler()
 				pid);
 #endif
 	}
+	errno = saved_errno;
 }
 
 /*
@@ -544,6 +562,31 @@ void cont_handler()
 	got_cont = 1;
 }
 
+/*
+ *	Dump core. Returns 0 if we are the child, so that the caller
+ *	can return if it is a signal handler - SIGSEGV is blocked in
+ *	the handler, so it will be raised when the handler returns.
+ */
+int coredump(void)
+{
+	static int	dumped = 0;
+	struct rlimit	rlim;
+
+	if (dumped) return 1;
+	dumped = 1;
+
+	if (fork() != 0) return 1;
+
+	rlim.rlim_cur = RLIM_INFINITY;
+	rlim.rlim_max = RLIM_INFINITY;
+	setrlimit(RLIMIT_CORE, &rlim);
+
+	chdir("/");
+	signal(SIGSEGV, SIG_DFL);
+	raise(SIGSEGV);
+	return 0;
+}
+
 /*
  *	OOPS: segmentation violation!
  *	If we have the info, print where it occured.
@@ -553,18 +596,26 @@ void cont_handler()
 void segv_handler(int sig, struct sigcontext ctx)
 {
 	char	*p = "";
+	int	saved_errno = errno;
 
 	if ((void *)ctx.eip >= (void *)do_sleep &&
 	    (void *)ctx.eip < (void *)main)
 		p = " (code)";
-	log(L_VB, "PANIC: segmentation violation at %p%s! sleeping for 30 seconds.", p);
-	do_sleep(30);
+	log(L_VB, "PANIC: segmentation violation at %p%s! "
+		  "sleeping for 30 seconds.", (void *)ctx.eip, p);
+	if (coredump() != 0)
+		do_sleep(30);
+	errno = saved_errno;
 }
 #else
 void segv_handler()
 {
+	int	saved_errno = errno;
+
 	log(L_VB, "PANIC: segmentation violation! sleeping for 30 seconds.");
-	do_sleep(30);
+	if (coredump() != 0)
+		do_sleep(30);
+	errno = saved_errno;
 }
 #endif
 
@@ -573,9 +624,12 @@ void segv_handler()
  */
 void stop_handler()
 {
+	int	saved_errno = errno;
+
 	got_cont = 0;
 	while(!got_cont) pause();
 	got_cont = 0;
+	errno = saved_errno;
 }
 
 /*
@@ -599,7 +653,7 @@ void set_term(int how)
 		/*
 		 *	Save the terminal settings.
 		 */
-		(void) ioctl(fd, TCGETS, &tty);
+		(void) tcgetattr(fd, &tty);
 		if ((fd2 = open(IOSAVE, O_WRONLY|O_CREAT|O_TRUNC, 0600)) < 0) {
 			if (errno != EROFS)
 				log(L_VB, "can't open(%s, O_WRONLY): %s",
@@ -627,21 +681,21 @@ void set_term(int how)
 		/*
 		 *	No old settings - create some resonable defaults.
 		 */
-		(void) ioctl(fd, TCGETS, &tty);
+		(void) tcgetattr(fd, &tty);
 
 		tty.c_cflag &= CBAUD|CBAUDEX|CSIZE|CSTOPB|PARENB|PARODD;
 		tty.c_cflag |= HUPCL|CLOCAL;
 
-		tty.c_cc[VINTR]  = 3;
-		tty.c_cc[VQUIT]  = 28;
+		tty.c_cc[VINTR]  = 3;	/* ctrl('c') */
+		tty.c_cc[VQUIT]  = 28;	/* ctrl('\\') */
 		tty.c_cc[VERASE] = 127;
-		tty.c_cc[VKILL]  = 24;
-		tty.c_cc[VEOF]   = 4;
+		tty.c_cc[VKILL]  = 24;	/* ctrl('x') */
+		tty.c_cc[VEOF]   = 4;	/* ctrl('d') */
 		tty.c_cc[VTIME]  = 0;
 		tty.c_cc[VMIN]   = 1;
-		tty.c_cc[VSTART] = 17;
-		tty.c_cc[VSTOP]  = 19;
-		tty.c_cc[VSUSP]  = 26;
+		tty.c_cc[VSTART] = 17;	/* ctrl('q') */
+		tty.c_cc[VSTOP]  = 19;	/* ctrl('s') */
+		tty.c_cc[VSUSP]  = 26;	/* ctrl('z') */
 	}
 
 	/*
@@ -654,7 +708,7 @@ void set_term(int how)
 	/*
 	 *	Now set the terminal line.
 	 */
-	(void) ioctl(fd, TCSETS, &tty);
+	(void) tcsetattr(fd, TCSADRAIN, &tty);
 	(void) close(fd);
 }
 
@@ -680,7 +734,7 @@ void log(int loglevel, char *s, ...)
 	char buf[256];
 
 	va_start(va_alist, s);
-	vsprintf(buf, s, va_alist);
+	vsnprintf(buf, sizeof(buf), s, va_alist);
 	va_end(va_alist);
 
 	if (loglevel & L_SY) {
@@ -735,8 +789,10 @@ int spawn(CHILD *ch, int *res)
   char i_cons[32];		/* console device. */
   pid_t pid, pgrp;		/* child, console process group. */
   sigset_t nmask, omask;	/* For blocking SIGCHLD */
+  struct sigaction sa;
 
   *res = -1;
+  buf[sizeof(buf) - 1] = 0;
 
   /* Skip '+' if it's there */
   if (proc[0] == '+') proc++;
@@ -794,12 +850,12 @@ int spawn(CHILD *ch, int *res)
   	args[1] = SHELL;
   	args[2] = "-c";
   	strcpy(buf, "exec ");
-  	strcat(buf, proc);
+  	strncat(buf, proc, sizeof(buf) - strlen(buf) - 1);
   	args[3] = buf;
   	args[4] = NULL;
   } else {
 	/* Split up command line arguments */
-  	strcpy(buf, proc);
+  	strncpy(buf, proc, sizeof(buf) - 1);
   	ptr = buf;
   	for(f = 1; f < 15; f++) {
   		/* Skip white space */
@@ -843,7 +899,7 @@ int spawn(CHILD *ch, int *res)
   		sigprocmask(SIG_SETMASK, &omask, NULL);
 
 		/* Now set RUNLEVEL and PREVLEVEL */
-		sprintf(i_cons, "CONSOLE=%s", console_dev);
+		snprintf(i_cons, sizeof(i_cons), "CONSOLE=%s", console_dev);
 		i_lvl[9]   = thislevel;
 		i_prev[10] = prevlevel;
 		putenv(i_lvl);
@@ -876,15 +932,15 @@ int spawn(CHILD *ch, int *res)
 				/*
 				 *	Ignore keyboard signals etc.
 				 */
-				signal(SIGINT, SIG_IGN);
-				signal(SIGQUIT, SIG_IGN);
-				signal(SIGTSTP, SIG_IGN);
-				signal(SIGCHLD, SIG_DFL);
+				SETSIG(sa, SIGINT, SIG_IGN, SA_RESTART);
+				SETSIG(sa, SIGTSTP, SIG_IGN, SA_RESTART);
+				SETSIG(sa, SIGQUIT, SIG_IGN, SA_RESTART);
+				SETSIG(sa, SIGCHLD, SIG_DFL, SA_RESTART);
 
 				while(waitpid(pid, &st, 0) != pid)
 					;
 				pgrp = 0;
-				(void)ioctl(f, TIOCGPGRP, &pgrp);
+				pgrp = tcgetpgrp(f);
 				if (pgrp != getpid())
 					exit(0);
 				/*
@@ -917,9 +973,11 @@ int spawn(CHILD *ch, int *res)
 			dup(f);
 			setsid();
 		}
+
   		/* Reset all the signals */
-  		for(f = 1; f < NSIG; f++) signal(f, SIG_DFL);
+  		for(f = 1; f < NSIG; f++) SETSIG(sa, f, SIG_DFL, SA_RESTART);
   		execvp(args[1], args + 1);
+
 		/*
 		 *	Is this a bug in execvp? It does _not_ execute shell
 		 *	scripts (/etc/rc !), so we try again with
@@ -929,7 +987,7 @@ int spawn(CHILD *ch, int *res)
   			args[1] = SHELL;
   			args[2] = "-c";
   			strcpy(buf, "exec ");
-  			strcat(buf, proc);
+  			strncat(buf, proc, sizeof(buf) - strlen(buf) - 1);
   			args[3] = buf;
   			args[4] = NULL;
 			execvp(args[1], args + 1);
@@ -1059,7 +1117,7 @@ void read_inittab(void)
 	/*
 	 *	Add single user shell entry at the end.
 	 */
-	if (fp == NULL || fgets(buf, 255, fp) == NULL) {
+	if (fp == NULL || fgets(buf, sizeof(buf), fp) == NULL) {
 		done = 1;
 		/*
 		 *	See if we have a single user entry.
@@ -1067,7 +1125,7 @@ void read_inittab(void)
 		for(old = newFamily; old; old = old->next)
 			if (strcmp(old->rlevel, "S") == 0) break;
 		if (old == NULL)
-			sprintf(buf, "~~:S:wait:%s\n", SHELL);
+			snprintf(buf, sizeof(buf), "~~:S:wait:%s\n", SHELL);
 		else
 			continue;
 	}
@@ -1155,13 +1213,13 @@ void read_inittab(void)
 	 */
 	ch->action = actionNo;
 	strncpy(ch->id, id, sizeof(utproto.ut_id) + 1); /* Hack for different libs. */
-	strncpy(ch->process, process, 127);
+	strncpy(ch->process, process, sizeof(ch->process) - 1);
 	if (rlevel[0]) {
-		for(f = 0; f < 11 && rlevel[f]; f++) {
+		for(f = 0; f < sizeof(rlevel) - 1 && rlevel[f]; f++) {
 			ch->rlevel[f] = rlevel[f];
 			if (ch->rlevel[f] == 's') ch->rlevel[f] = 'S';
 		}
-		strncpy(ch->rlevel, rlevel, 11);
+		strncpy(ch->rlevel, rlevel, sizeof(ch->rlevel) - 1);
 	} else {
 		strcpy(ch->rlevel, "0123456789");
 		if (ISPOWER(ch->action))
@@ -1457,9 +1515,10 @@ void start_if_needed(void)
  */
 int ask_runlevel()
 {
-	int lvl = -1;
-	char buf[8];
-	int fd;
+	const char	prompt[] = "\nEnter runlevel: ";
+	char		buf[8];
+	int		lvl = -1;
+	int		fd;
 
 	set_term(0);
 	fd = console_open(O_RDWR|O_NOCTTY);
@@ -1467,9 +1526,9 @@ int ask_runlevel()
 	if (fd < 0) return('S');
 
 	while(!strchr("0123456789S", lvl)) {
-  		write(fd, "\nEnter runlevel: ", 17);
+  		write(fd, prompt, sizeof(prompt) - 1);
 		buf[0] = 0;
-  		read(fd, buf, 8);
+  		read(fd, buf, sizeof(buf));
   		if (buf[0] != 0 && (buf[1] == '\r' || buf[1] == '\n'))
 			lvl = buf[0];
 		if (islower(lvl)) lvl = toupper(lvl);
@@ -1664,13 +1723,13 @@ void fail_check(void)
 /* Set all 'Fail' timers to 0 */
 void fail_cancel(void)
 {
-  CHILD *ch;
+	CHILD *ch;
 
-  for(ch = family; ch; ch = ch->next) {
-	ch->count = 0;
-	ch->tm = 0;
-	ch->flags &= ~FAILING;
-  }
+	for(ch = family; ch; ch = ch->next) {
+		ch->count = 0;
+		ch->tm = 0;
+		ch->flags &= ~FAILING;
+	}
 }
 
 /*
@@ -2027,8 +2086,9 @@ void boot_transitions()
 			set_term(1);
 
 			/* Write a boot record. */
-			wrote_reboot = 0;
-			write_wtmp("reboot", "~~", 0, BOOT_TIME, "~");
+			wrote_utmp_reboot = 0;
+			wrote_wtmp_reboot = 0;
+			write_utmp_wtmp("reboot", "~~", 0, BOOT_TIME, "~");
 
   			/* Get our run level */
   			newlevel = dfl_level ? dfl_level : get_init_default();
@@ -2231,9 +2291,11 @@ int init_main()
 	 * Fork so we can debug the init process.
 	 */
 	if ((f = fork()) > 0) {
+		static const char killmsg[] = "PRNT: init killed.\r\n";
+
 		while(wait(&st) != f)
 			;
-		write(1, "PRNT: init killed.\r\n", 20);
+		write(1, killmsg, sizeof(killmsg) - 1);
 		while(1) pause();
 	}
 #endif
@@ -2253,32 +2315,20 @@ int init_main()
 	 *	Ignore all signals.
 	 */
 	for(f = 1; f <= NSIG; f++)
-		signal(f, SIG_IGN);
+		SETSIG(sa, f, SIG_IGN, SA_RESTART);
   }
 
-  SETSIG(sa, SIGALRM,  signal_handler);
-  SETSIG(sa, SIGHUP,   signal_handler);
-  SETSIG(sa, SIGINT,   signal_handler);
-  SETSIG(sa, SIGCHLD,  chld_handler);
-  SETSIG(sa, SIGPWR,   signal_handler);
-  SETSIG(sa, SIGWINCH, signal_handler);
-  SETSIG(sa, SIGUSR1,  signal_handler);
-  SETSIG(sa, SIGSTOP,  stop_handler);
-  SETSIG(sa, SIGTSTP,  stop_handler);
-  SETSIG(sa, SIGCONT,  cont_handler);
-  SETSIG(sa, SIGSEGV,  (void(*)(int))segv_handler);
-
-  /*
-   *	NOTE: instead of calling siginterrupt, we could just as
-   *	well install the signal handler without SA_RESTART in the
-   *	first place.
-   */
-  siginterrupt (SIGALRM,  1);
-  siginterrupt (SIGHUP,   1);
-  siginterrupt (SIGINT,   1);
-  siginterrupt (SIGPWR,   1);
-  siginterrupt (SIGUSR1,  1);
-  siginterrupt (SIGWINCH, 1);
+  SETSIG(sa, SIGALRM,  signal_handler, 0);
+  SETSIG(sa, SIGHUP,   signal_handler, 0);
+  SETSIG(sa, SIGINT,   signal_handler, 0);
+  SETSIG(sa, SIGCHLD,  chld_handler, SA_RESTART);
+  SETSIG(sa, SIGPWR,   signal_handler, 0);
+  SETSIG(sa, SIGWINCH, signal_handler, 0);
+  SETSIG(sa, SIGUSR1,  signal_handler, 0);
+  SETSIG(sa, SIGSTOP,  stop_handler, SA_RESTART);
+  SETSIG(sa, SIGTSTP,  stop_handler, SA_RESTART);
+  SETSIG(sa, SIGCONT,  cont_handler, SA_RESTART);
+  SETSIG(sa, SIGSEGV,  (void (*)(int))segv_handler, SA_RESTART);
 
   console_init();
 
@@ -2305,18 +2355,18 @@ int init_main()
   	/*
 	 *	Say hello to the world
 	 */
-  	log(L_CO, bootmsg);
+  	log(L_CO, bootmsg, "booting");
 
   	/*
 	 *	See if we have to start an emergency shell.
 	 */
 	if (emerg_shell) {
-		SETSIG(sa, SIGCHLD, SIG_DFL);
+		SETSIG(sa, SIGCHLD, SIG_DFL, SA_RESTART);
 		if (spawn(&ch_emerg, &f) > 0) {
 			while(wait(&st) != f)
 				;
 		}
-  		SETSIG(sa, SIGCHLD,  chld_handler);
+  		SETSIG(sa, SIGCHLD,  chld_handler, SA_RESTART);
   	}
 
   	/*
@@ -2329,7 +2379,7 @@ int init_main()
 	/*
 	 *	Restart: unblock signals and let the show go on
 	 */
-	log(L_CO, bootmsg);
+	log(L_CO, bootmsg, "reloading");
 	sigfillset(&sgt);
 	sigprocmask(SIG_UNBLOCK, &sgt, NULL);
   }
@@ -2388,10 +2438,11 @@ void Usage(char *s)
  */
 int main(int argc, char *argv[])
 {
-  struct init_request request;
-  char *p;
-  FILE *fp;
-  int f, fd;
+  struct init_request	request;
+  struct sigaction	sa;
+  FILE			*fp;
+  char			*p;
+  int			f, fd;
 
   /* Get my own name */
   if ((p = strrchr(argv[0], '/')) != NULL)
@@ -2433,10 +2484,13 @@ int main(int argc, char *argv[])
 			putenv("AUTOBOOT=YES");
 		else if (!strcmp(argv[f], "-b") || !strcmp(argv[f], "emergency"))
 			emerg_shell = 1;
-		else if (strchr("0123456789sS", argv[f][0])
+		else if (!strcmp(argv[f], "-z")) {
+			/* Ignore -z xxx */
+			if (argv[f + 1]) f++;
+		} else if (strchr("0123456789sS", argv[f][0])
 			&& strlen(argv[f]) == 1)
 			dfl_level = argv[f][0];
-		/* "init u" in the very beginning makes no sence */
+		/* "init u" in the very beginning makes no sense */
 		if (dfl_level == 's') dfl_level = 'S';
 		maxproclen += strlen(argv[f]) + 1;
 	}
@@ -2480,8 +2534,7 @@ int main(int argc, char *argv[])
   request.sleeptime = sltime;
 
   /* Make sure we don't hang on opening /etc/init.fifo */
-  signal(SIGALRM, signal_handler);
-  siginterrupt (SIGALRM, 1);
+  SETSIG(sa, SIGALRM, signal_handler, 0);
   alarm(3);
   if ((fd = open(INIT_FIFO, O_WRONLY)) >= 0 &&
 	    write(fd, &request, sizeof(request)) == sizeof(request)) {
