@@ -147,7 +147,7 @@ int readproc()
   }
 
   /* Free the already existing process list. */
-  n = NULL;
+  n = plist;
   for(p = plist; n; p = n) {
 	n = p->next;
 	if (p->fullname) free(p->fullname);
