@@ -6,10 +6,10 @@
  *
  * Author:	Miquel van Smoorenburg, miquels@cistron.nl
  *
- * Version:	@(#)last  2.76  27-Dec-1998  miquels@cistron.nl
+ * Version:	@(#)last  2.78  24-Nov-1999  miquels@cistron.nl
  *
  *		This file is part of the sysvinit suite,
- *		Copyright 1991-1998 Miquel van Smoorenburg.
+ *		Copyright 1991-1999 Miquel van Smoorenburg.
  *
  *		This program is free software; you can redistribute it and/or
  *		modify it under the terms of the GNU General Public License
@@ -22,6 +22,7 @@
 #include <sys/fcntl.h>
 #include <time.h>
 #include <stdio.h>
+#include <ctype.h>
 #include <utmp.h>
 #include <errno.h>
 #include <malloc.h>
@@ -39,11 +40,11 @@
 #  define SHUTDOWN_TIME 254
 #endif
 
-char *Version = "@(#) last 2.76 27-Dec-1998 miquels";
+char *Version = "@(#) last 2.78 24-Nov-1999 miquels";
 
 #define CHOP_DOMAIN	0	/* Define to chop off local domainname. */
 #define NEW_UTMP	1	/* Fancy & fast utmp read code. */
-#define UCHUNKSIZE	16384	/* Howmuch we read at once. */
+#define UCHUNKSIZE	16384	/* How much we read at once. */
 
 /* Double linked list of struct utmp's */
 struct utmplist {
@@ -66,6 +67,7 @@ int recsdone = 0;	/* Number of records listed */
 int showhost = 1;	/* Show hostname too? */
 int altlist = 0;	/* Show hostname at the end. */
 int usedns = 0;		/* Use DNS to lookup the hostname. */
+int useip = 0;		/* Print IP address in number format */
 int oldfmt = 0;		/* Use old libc5 format? */
 char **show = NULL;	/* What do they want us to show */
 char *ufile;		/* Filename of this file */
@@ -324,11 +326,12 @@ int dns_lookup(char *result, char *org, unsigned int ip)
  */
 int list(struct utmp *p, time_t t, int what)
 {
+	struct in_addr in;
 	char logintime[32];
 	char logouttime[32];
 	char length[32];
 	char final[128];
-	char utline[12];
+	char utline[UT_LINESIZE+1];
 	char domain[256];
 	time_t secs, tmp;
 	int mins, hours, days;
@@ -337,19 +340,22 @@ int list(struct utmp *p, time_t t, int what)
 	/*
 	 *	uucp and ftp have special-type entries
 	 */
-	strncpy(utline, p->ut_line, sizeof(utline));
-	if (strncmp(utline, "ftp", 3) == 0)  utline[3] = 0;
-	if (strncmp(utline, "uucp", 4) == 0) utline[4] = 0;
+	strncpy(utline, p->ut_line, UT_LINESIZE);
+	utline[UT_LINESIZE - 1] = 0;
+	if (strncmp(utline, "ftp", 3) == 0 && isdigit(utline[3]))
+		utline[3] = 0;
+	if (strncmp(utline, "uucp", 4) == 0 && isdigit(utline[4]))
+		utline[4] = 0;
 
 	/*
 	 *	Is this something we wanna show?
 	 */
 	if (show) {
 		for (walk = show; *walk; walk++) {
-			if (strncmp(p->ut_name, *walk, 8) == 0 ||
-			    strncmp(utline, *walk, 12) == 0 ||
+			if (strncmp(p->ut_name, *walk, UT_NAMESIZE) == 0 ||
+			    strcmp(utline, *walk) == 0 ||
 			    (strncmp(utline, "tty", 3) == 0 &&
-			     strncmp(utline + 3, *walk, 9) == 0)) break;
+			     strcmp(utline + 3, *walk) == 0)) break;
 		}
 		if (*walk == NULL) return 0;
 	}
@@ -395,7 +401,10 @@ int list(struct utmp *p, time_t t, int what)
 	 */
 	if (usedns)
 		dns_lookup(domain, p->ut_host, p->ut_addr);
-	else {
+	if (useip) {
+		in.s_addr = p->ut_addr;
+		strcpy(domain, inet_ntoa(in));
+	} else {
 		strncpy(domain, p->ut_host, UT_HOSTSIZE);
 		domain[UT_HOSTSIZE-1] = 0;
 	}
@@ -477,7 +486,7 @@ int main(int argc, char **argv)
   progname = mybasename(argv[0]);
 
   /* Process the arguments. */
-  while((c = getopt(argc, argv, "f:n:Rxado0123456789")) != EOF)
+  while((c = getopt(argc, argv, "f:n:Rxadio0123456789")) != EOF)
     switch(c) {
 	case 'R':
 		showhost = 0;
@@ -502,6 +511,9 @@ int main(int argc, char **argv)
 	case 'd':
 		usedns++;
 		break;
+	case 'i':
+		useip++;
+		break;
 	case 'a':
 		altlist++;
 		break;
@@ -670,8 +682,8 @@ int main(int argc, char **argv)
 			c = 0;
 			for (p = utmplist; p; p = next) {
 				next = p->next;
-				if (strncmp(p->ut.ut_line, ut.ut_line, 12) == 0)
-				{
+				if (strncmp(p->ut.ut_line, ut.ut_line,
+				    UT_LINESIZE) == 0) {
 					/* Show it */
 					if (c == 0) {
 						quit = list(&ut, p->ut.ut_time,
