@@ -11,7 +11,7 @@
  *		encrypted root password is "x" the shadow
  *		password will be used.
  *
- * Version:	@(#)sulogin 2.76 20 Nov-1998 miquels@cistron.nl
+ * Version:	@(#)sulogin 2.76-4 24-Apr-1999 miquels@cistron.nl
  *
  */
 
@@ -37,7 +37,7 @@
 #define SHADOW	"/etc/shadow"
 #define BINSH	"/bin/sh"
 
-char *Version = "@(#)sulogin 2.76 20-Nov-1998 miquels@cistron.nl";
+char *Version = "@(#)sulogin 2.76-4 24-apr-1999 miquels@cistron.nl";
 
 int timeout = 0;
 int profile = 0;
@@ -87,7 +87,6 @@ void fixtty(void)
  */
 void alrm_handler()
 {
-	signal(SIGALRM, alrm_handler);
 }
 
 /*
@@ -150,7 +149,7 @@ struct passwd *getrootpwent()
 	char *p;
 
 	/*
-	 *	Fill out some defaults in advance.
+	 *	Set defaults.
 	 */
 	pwd.pw_name = "root";
 	pwd.pw_passwd = "";
@@ -199,7 +198,7 @@ struct passwd *getrootpwent()
 	 */
 	strcpy(pwd.pw_passwd, "");
 	if ((fp = fopen(SHADOW, "r")) == NULL) {
-		fprintf(stderr, "%s: root password invalid\n", PASSWD);
+		fprintf(stderr, "%s: root password garbled\n", PASSWD);
 		return &pwd;
 	}
 	while((p = fgets(sline, 256, fp)) != NULL) {
@@ -220,7 +219,7 @@ struct passwd *getrootpwent()
 		strcpy(pwd.pw_passwd, "");
 	}
 	if (!valid(pwd.pw_passwd)) {
-		fprintf(stderr, "%s: root password invalid\n", SHADOW);
+		fprintf(stderr, "%s: root password garbled\n", SHADOW);
 		strcpy(pwd.pw_passwd, ""); }
 	return &pwd;
 }
@@ -231,6 +230,7 @@ struct passwd *getrootpwent()
  */
 char *getpasswd()
 {
+	struct sigaction sa;
 	struct termios old, tty;
 	static char pass[128];
 	char *ret = pass;
@@ -247,7 +247,12 @@ char *getpasswd()
 	tcsetattr(0, TCSANOW, &tty);
 
 	pass[sizeof(pass) - 1] = 0;
+
+	sa.sa_handler = alrm_handler;
+	sa.sa_flags = 0;
+	sigaction(SIGALRM, &sa, NULL);
 	if (timeout) alarm(timeout);
+
 	if (read(0, pass, sizeof(pass) - 1) <= 0)
 		ret = NULL;
 	else {
@@ -344,8 +349,10 @@ int main(int argc, char **argv)
 			break;
 	}
 
-	siginterrupt (SIGALRM, 1);
-	signal(SIGALRM, alrm_handler);
+	if (geteuid() != 0) {
+		fprintf(stderr, "sulogin: only root can run sulogin.\n");
+		exit(1);
+	}
 
 	/*
 	 *	See if we need to open an other tty device.
